@@ -6,6 +6,7 @@
 # GNU General Public License version 2, incorporated herein by reference.
 
 import os
+import re
 import sys
 import shlex
 import time
@@ -22,7 +23,7 @@ except ImportError:
 demandimport.enable()
 
 from mercurial import ui, util, extensions, match, bundlerepo, url, cmdutil
-from mercurial import dispatch, encoding, templatefilters, filemerge
+from mercurial import dispatch, encoding, templatefilters, filemerge, error
 
 _encoding = encoding.encoding
 _encodingmode = encoding.encodingmode
@@ -32,7 +33,6 @@ _fallbackencoding = encoding.fallbackencoding
 _extensions_blacklist = ('color', 'pager', 'progress')
 
 from tortoisehg.util import paths
-from tortoisehg.util.i18n import _
 from tortoisehg.util.hgversion import hgversion
 
 def tounicode(s):
@@ -42,6 +42,8 @@ def tounicode(s):
     Based on mercurial.util.tolocal().
     Return 'unicode' type string.
     """
+    if s is None:
+        return None
     if isinstance(s, unicode):
         return s
     for e in ('utf-8', _encoding):
@@ -51,12 +53,34 @@ def tounicode(s):
             pass
     return s.decode(_fallbackencoding, 'replace')
 
+def fromunicode(s, errors='strict'):
+    """
+    Convert the encoding of string from Unicode to MBCS.
+
+    Return 'str' type string.
+
+    If you don't want an exception for conversion failure,
+    specify errors='replace'.
+    """
+    if s is None:
+        return None
+    s = unicode(s)  # s can be QtCore.QString
+    for enc in (_encoding, _fallbackencoding):
+        try:
+            return s.encode(enc)
+        except UnicodeEncodeError:
+            pass
+
+    return s.encode(_encoding, errors)  # last ditch
+
 def toutf(s):
     """
     Convert the encoding of string from MBCS to UTF-8.
 
     Return 'str' type string.
     """
+    if s is None:
+        return None
     return tounicode(s).encode('utf-8').replace('\0','')
 
 def fromutf(s):
@@ -65,6 +89,8 @@ def fromutf(s):
 
     Return 'str' type string.
     """
+    if s is None:
+        return None
     try:
         return s.decode('utf-8').encode(_encoding)
     except (UnicodeDecodeError, UnicodeEncodeError):
@@ -160,6 +186,42 @@ def getfilteredtags(repo):
             filtered.append(tag)
     return filtered
 
+def getrawctxtags(changectx):
+    '''Returns the tags for changectx, converted to UTF-8 but
+    unfiltered for hidden tags'''
+    value = [toutf(tag) for tag in changectx.tags()]
+    if len(value) == 0:
+        return None
+    return value
+
+def getctxtags(changectx):
+    '''Returns all unhidden tags for changectx, converted to UTF-8'''
+    value = getrawctxtags(changectx)
+    if value:
+        htlist = gethidetags(changectx._repo.ui)
+        tags = [tag for tag in value if tag not in htlist]
+        if len(tags) == 0:
+            return None
+        return tags
+    return None
+
+def getmqpatchtags(repo):
+    '''Returns all tag names used by MQ patches, or []'''
+    if hasattr(repo, 'mq'):
+        repo.mq.parse_series()
+        return repo.mq.series[:]
+    else:
+        return []
+
+def getcurrentqqueue(repo):
+    """Return the name of the current patch queue."""
+    if not hasattr(repo, 'mq'):
+        return None
+    cur = os.path.basename(repo.mq.path)
+    if cur.startswith('patches-'):
+        cur = cur[8:]
+    return cur
+
 def diffexpand(line):
     'Expand tabs in a line of diff/patch text'
     if _tabwidth is None:
@@ -187,28 +249,55 @@ def getfontconfig(_ui=None):
 
 def invalidaterepo(repo):
     repo.dirstate.invalidate()
+    for attr in ('_bookmarks', '_bookmarkcurrent'):
+        if attr in repo.__dict__:
+            delattr(repo, attr)
     if isinstance(repo, bundlerepo.bundlerepository):
         # Work around a bug in hg-1.3.  repo.invalidate() breaks
         # overlay bundlerepos
         return
     repo.invalidate()
-    # way _bookmarks / _bookmarkcurrent cached changed
-    # from 1.4 to 1.5...
-    for cachedAttr in ('_bookmarks', '_bookmarkcurrent'):
-        # Check if it's a property or normal value...
-        if is_descriptor(repo, cachedAttr):
-            # The very act of calling hasattr would
-            # re-cache the property, so just assume it's
-            # already cached, and catch the error if it wasn't.
-            try:
-                delattr(repo, cachedAttr)
-            except AttributeError:
-                pass
-        elif hasattr(repo, cachedAttr):
-            setattr(repo, cachedAttr, None)
     if 'mq' in repo.__dict__: #do not create if it does not exist
         repo.mq.invalidate()
 
+def enabledextensions():
+    """Return the {name: shortdesc} dict of enabled extensions
+
+    shortdesc is in local encoding.
+    """
+    return extensions.enabled()[0]
+
+def allextensions():
+    """Return the {name: shortdesc} dict of known extensions
+
+    shortdesc is in local encoding.
+    """
+    enabledexts = enabledextensions()
+    disabledexts = extensions.disabled()[0]
+    exts = (disabledexts or {}).copy()
+    exts.update(enabledexts)
+    return exts
+
+def validateextensions(enabledexts):
+    """Report extensions which should be disabled
+
+    Returns the dict {name: message} of extensions expected to be disabled.
+    message is 'utf-8'-encoded string.
+    """
+    from tortoisehg.util.i18n import _  # avoid cyclic dependency
+    exts = {}
+    if os.name != 'posix':
+        exts['inotify'] = _('inotify is not supported on this platform')
+    if 'win32text' in enabledexts:
+        exts['eol'] = _('eol is incompatible with win32text')
+    if 'eol' in enabledexts:
+        exts['win32text'] = _('win32text is incompatible with eol')
+    if 'perfarce' in enabledexts:
+        exts['hgsubversion'] = _('hgsubversion is incompatible with perfarce')
+    if 'hgsubversion' in enabledexts:
+        exts['perfarce'] = _('perfarce is incompatible with hgsubversion')
+    return exts
+
 def loadextension(ui, name):
     # Between Mercurial revisions 1.2 and 1.3, extensions.load() stopped
     # calling uisetup() after loading an extension.  This could do
@@ -282,9 +371,11 @@ def mergetools(ui, values=None):
     'returns the configured merge tools and the internal ones'
     if values == None:
         values = []
+    seen = values[:]
     for key, value in ui.configitems('merge-tools'):
         t = key.split('.')[0]
-        if t not in values:
+        if t not in seen:
+            seen.append(t)
             # Ensure the tool is installed
             if filemerge._findtool(ui, t):
                 values.append(t)
@@ -348,6 +439,21 @@ def difftools(ui):
     return tools
 
 
+_funcre = re.compile('\w')
+def getchunkfunction(data, linenum):
+    """Return the function containing the chunk at linenum.
+
+    Stolen from mercurial/mdiff.py.
+    """
+    # Walk backwards starting from the line before the chunk
+    # to find a line starting with an alphanumeric char.
+    for x in xrange(int(linenum) - 2, -1, -1):
+        t = data[x].rstrip()
+        if _funcre.match(t):
+            return ' ' + t[:40]
+    return None
+
+
 def hgcmd_toq(q, label, args):
     '''
     Run an hg command in a background thread, pipe all output to a Queue
@@ -403,6 +509,36 @@ def username(user):
         author = util.shortuser(user)
     return author
 
+def user(ctx):
+    '''
+    Get the username of the change context. Does not abort and just returns
+    an empty string if ctx is a working context and no username has been set
+    in mercurial's config.
+    '''
+    try:
+        user = ctx.user()
+    except error.Abort:
+        if ctx._rev is not None:
+            raise
+        # ctx is a working context and probably no username has
+        # been configured in mercurial's config
+        user = ''
+    return user
+
+def get_revision_desc(fctx, curpath=None):
+    """return the revision description as a string"""
+    author = tounicode(username(fctx.user()))
+    rev = fctx.linkrev()
+    # If the source path matches the current path, don't bother including it.
+    if curpath and curpath == fctx.path():
+        source = ''
+    else:
+        source = '(%s)' % fctx.path()
+    date = age(fctx.date())
+    l = tounicode(fctx.description()).replace('\0', '').splitlines()
+    summary = l and l[0] or ''
+    return '%s@%s%s:%s "%s"' % (author, rev, source, date, summary)
+
 def validate_synch_path(path, repo):
     '''
     Validate the path that must be used to sync operations (pull,
@@ -416,74 +552,21 @@ def validate_synch_path(path, repo):
             return_path = path_aux
     return return_path
 
-def get_repo_bookmarks(repo, values=False):
-    """
-    Will return the bookmarks for the given repo if the
-    bookmarks extension is loaded.
-    
-    By default, returns a list of bookmark names; if
-    values is True, returns a dict mapping names to 
-    nodes.
-    
-    If the extension is not loaded, returns an empty
-    list/dict.
-    """
-    try:
-        bookmarks = extensions.find('bookmarks')
-    except KeyError:
-        return values and {} or []
-    if bookmarks:
-        # Bookmarks changed from 1.4 to 1.5...
-        if hasattr(bookmarks, 'parse'):
-            marks = bookmarks.parse(repo)
-        elif hasattr(repo, '_bookmarks'):
-            marks = repo._bookmarks
-        else:
-            marks = {}
-    else:
-        marks = {}
-            
-    if values:
-        return marks
-    else:
-        return marks.keys()
-    
-def get_repo_bookmarkcurrent(repo):
-    """
-    Will return the current bookmark for the given repo
-    if the bookmarks extension is loaded, and the
-    track.current option is on.
-    
-    If the extension is not loaded, or track.current
-    is not set, returns None
-    """
-    try:
-        bookmarks = extensions.find('bookmarks')
-    except KeyError:
-        return None
-    if bookmarks and repo.ui.configbool('bookmarks', 'track.current'):
-        # Bookmarks changed from 1.4 to 1.5...
-        if hasattr(bookmarks, 'current'):
-            return bookmarks.current(repo)
-        elif hasattr(repo, '_bookmarkcurrent'):
-            return repo._bookmarkcurrent
-    return None
-
 def is_rev_current(repo, rev):
     '''
     Returns True if the revision indicated by 'rev' is the current
     working directory parent.
-    
+
     If rev is '' or None, it is assumed to mean 'tip'.
     '''
     if rev in ('', None):
         rev = 'tip'
     rev = repo.lookup(rev)
     parents = repo.parents()
-    
+
     if len(parents) > 1:
         return False
-    
+
     return rev == parents[0].node()
 
 def is_descriptor(obj, attr):
@@ -499,15 +582,15 @@ def is_descriptor(obj, attr):
         if attr in cls.__dict__:
             return hasattr(cls.__dict__[attr], '__get__')
     return None
-    
+
 def export(repo, revs, template='hg-%h.patch', fp=None, switch_parent=False,
            opts=None):
     '''
     export changesets as hg patches.
-    
+
     Mercurial moved patch.export to cmdutil.export after version 1.5
     (change e764f24a45ee in mercurial).
-    '''   
+    '''
 
     try:
         return cmdutil.export(repo, revs, template, fp, switch_parent, opts)
